#!/bin/sh
#
# faked ispell for openwebmail debug
#
echo "@(#) International Ispell Version 3.1.20 faked"
while read line; do
   echo "& $line 3 0: aaa, bbb, ccc"
   echo ""
   echo $line >> /tmp/fakedspell.log
done
